# gta_utils.py @space_view3d_gta_tools
# 2011 ponz
script_info = "GTA Tools Utilities ( built:2012.02.19 )"

# ***** BEGIN GPL LICENSE BLOCK *****
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or ( at your option ) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation, 
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ***** END GPL LICENCE BLOCK *****


#####
# ToDo.
#  - 
#####

import bpy
import os
import time
import struct
from bpy.props import *
from mathutils import *
from math import *
#from rna_prop_ui import rna_idprop_ui_prop_get

import bgl
import blf

## Constants

## Classes

## Variables

## Functions
def get_root( obj, depth ):
	if None == obj.parent or 0 == depth:
		return obj
	else:
		return get_root( obj.parent, depth-1 )

def get_children( obj, children, depth ):
	children.append( obj )
	if 0 == len( obj.children ) or 0 == depth:
		return
	else:
		for child in obj.children:
			get_children( child, children, depth-1 )

def GetLinkedObjs( sel_objs ):
	sel_objs = list( set( sel_objs ) )
	recursive_depth = 10
	objs = []
	for obj in sel_objs:
		if None != obj:
			root = get_root( obj, recursive_depth )
			get_children( root, objs, recursive_depth )
	
	return list( set( objs ) )

def tgl_node():
	sel_objs = bpy.context.selected_objects
	sel_objs.append( bpy.context.scene.objects.active )
	objs = GetLinkedObjs( sel_objs )
	if 0 == len( objs ):
		bpy.context.scene.gta_tools.msg = "Error : No Active Object"
		bpy.context.scene.gta_tools.show_msg()
		return
	
	flg = True
	val = False
	for obj in objs:
		if 'EMPTY' == obj.type:
			if True == flg:
				val = not(obj.hide)
				flg = False
			obj.hide = val
			obj.hide_render = val
		
		elif 'ARMATURE' == obj.type:
			bones = []
			if   'OBJECT' == obj.mode: bones = obj.data.bones
			elif 'EDIT'   == obj.mode: bones = obj.data.edit_bones
			elif 'POSE'   == obj.mode:
				for pbone in obj.pose.bones: bones.append( pbone.bone )
			for bone in bones:
				if True == flg:
					val = not(bone.hide)
					flg = False
				bone.hide = val

def tgl_name():
	sel_objs = bpy.context.selected_objects
	sel_objs.append( bpy.context.scene.objects.active )
	objs = GetLinkedObjs( sel_objs )
	if 0 == len( objs ):
		bpy.context.scene.gta_tools.msg = "Error : No Active Object"
		bpy.context.scene.gta_tools.show_msg()
		return
	
	val = not( objs[0].show_name )
	for obj in objs:
		obj.show_name = val
		if 'ARMATURE' == obj.type: obj.data.show_names = val

def tgl_xray():
	sel_objs = bpy.context.selected_objects
	sel_objs.append( bpy.context.scene.objects.active )
	objs = GetLinkedObjs( sel_objs )
	if 0 == len( objs ):
		bpy.context.scene.gta_tools.msg = "Error : No Active Object"
		bpy.context.scene.gta_tools.show_msg()
		return
	
	flg = True
	val = False
	for obj in objs:
		if 'EMPTY' == obj.type or 'ARMATURE' == obj.type:
			if True == flg:
				val = not(obj.show_x_ray)
				flg = False
			obj.show_x_ray = val

def set_objsets_prop( mode, val ):
	util_props = bpy.context.scene.gta_tools.util_props
	sel_objs = bpy.context.selected_objects
	sel_objs.append( bpy.context.scene.objects.active )
	objs = GetLinkedObjs( sel_objs )
	if 0 == len( objs ):
		bpy.context.scene.gta_tools.msg = "Error : No Active Object"
		bpy.context.scene.gta_tools.show_msg()
		return
	tars = []
	filters = []
	
	if util_props.target_all:
		for obj in objs:
			tars.append( obj )
	
	else:
		if util_props.target_coll: filters.extend( ["bounding", "collision", "shadow"] )
		if util_props.target_vlo:  filters.append( "vlo" )
		if util_props.target_ok:   filters.append( "ok" )
		if util_props.target_dam:  filters.append( "dam" )
		for obj in objs:
			for filter in filters:
				if -1 != obj.name.lower().find( filter ):
					tars.append( obj )
	
	for tar in tars:
		try:
			# mode: 0 = show/hide, 1 = select/deselect
			if 0 == mode:
				tar.hide = val
				tar.hide_render = val
			elif 1 == mode:
				tar.select = val
		except: pass

def rm_unused():
	print( "-----\nRemove UnUsed Data")
	bpy.context.scene.gta_tools.set_time_ini()
	objtypedict = {
		'MESH'     : [ True  , bpy.data.meshes    ] ,
		'CURVE'    : [ True  , bpy.data.curves    ] ,
		'SURFACE'  : [ True  , bpy.data.curves    ] ,
		'META'     : [ True  , bpy.data.metaballs ] ,
		'FONT'     : [ True  , bpy.data.fonts     ] ,
		'ARMATURE' : [ True  , bpy.data.armatures ] ,
		'LATTICE'  : [ True  , bpy.data.lattices  ] ,
		'EMPTY'    : [ False , bpy.data.meshes    ] ,
		'CAMERA'   : [ True  , bpy.data.cameras   ] ,
		'LAMP'     : [ True  , bpy.data.lamps     ] }
	
	num_odat_removed = 0
	num_mats_removed = 0
	num_texs_removed = 0
	num_imgs_removed = 0
	num_acts_removed = 0
	num_grps_removed = 0
	
	for obj_key in objtypedict:
		data_item_key = obj_key
		if objtypedict[obj_key][0]:
			data_item_list = objtypedict[obj_key][1]
			print("-----\n" + str(data_item_key) )
			for data_item in data_item_list[:]:
				print( data_item )
				try:
					data_item_list.remove(data_item)
					print("  remove")
					num_odat_removed += 1
				except:
					print("  used")
	
	print("-----\nMaterials")
	for mat in bpy.data.materials[:]:
		print(mat)
		try:
			bpy.data.materials.remove(mat)
			print("  remove")
			num_mats_removed += 1
		except:
			print("  used")
	
	print("-----\nImage Texture")
	for tex in bpy.data.textures[:]:
		if 'IMAGE' == tex.type:
			print(tex)
			try:
				bpy.data.textures.remove(tex)
				print("  remove")
				num_texs_removed += 1
			except:
				print("  used")
	
	print("-----\nImage")
	for img in bpy.data.images[:]:
		if 'IMAGE' == img.type:
			print(img)
			try:
				bpy.data.images.remove(img)
				print("  remove")
				num_imgs_removed += 1
			except:
				print("  used")
	
	print("-----\nAction")
	for action in bpy.data.actions[:]:
		print(action)
		try:
			bpy.data.actions.remove(action)
			print( "  remove" )
			num_acts_removed += 1
		except:
			print( "  used" )
	
	print("-----\nGroup")
	for grp in bpy.data.groups[:]:
		print(grp)
		if 0 == len( grp.objects ):
			try:
				bpy.data.groups.remove(grp)
				print( "  remove" )
				num_grps_removed += 1
			except:
				print( "  used" )
		else:
			print( "  used" )
	
	bpy.context.scene.gta_tools.msg  = ( "--- Remove UnUsed ---\n" )
	bpy.context.scene.gta_tools.msg += ( "  Removed ObjData    = %d\n" %( num_odat_removed ) )
	bpy.context.scene.gta_tools.msg += ( "  Removed Materials  = %d\n" %( num_mats_removed ) )
	bpy.context.scene.gta_tools.msg += ( "  Removed Textures   = %d\n" %( num_texs_removed ) )
	bpy.context.scene.gta_tools.msg += ( "  Removed Images     = %d\n" %( num_imgs_removed ) )
	bpy.context.scene.gta_tools.msg += ( "  Removed Animations = %d\n" %( num_acts_removed ) )
	bpy.context.scene.gta_tools.msg += ( "  Removed Groups     = %d\n" %( num_grps_removed ) )
	bpy.context.scene.gta_tools.show_msg_elapsed()


def SetProps():
	global script_info
	print( "\n-----\n" + script_info + "\n-----" )
	bpy.context.scene.gta_tools.msg =  "--- Set Properties ---\n"
	
	util_props = bpy.context.scene.gta_tools.util_props
	objs = bpy.context.selected_objects
	for obj in objs:
		is_alp = False
		mats = obj.material_slots
		for mat in mats:
			texs = mat.material.texture_slots
			for texslot in texs:
				if None != texslot:
					if "SET" == util_props.normal_map:
						texslot.use_map_normal = True
						texslot.normal_factor  = util_props.normal_factor
					elif "DESET" == util_props.normal_map:
						texslot.use_map_normal = False
					
					if "CURRENT" != util_props.rename_alpha_objs:
						if texslot.use_map_alpha:
							is_alp = True
							break
			
			if "SET" == util_props.use_alpha:
				mat.material.use_transparency  = True
				mat.material.transparency_method = "RAYTRACE"
			elif "DESET" == util_props.use_alpha:
				mat.material.use_transparency  = False
				
			if "SET" == util_props.use_transparent_shadows:
				mat.material.use_transparent_shadows  = True
			elif "DESET" == util_props.use_transparent_shadows:
				mat.material.use_transparent_shadows  = False
			
		if "CURRENT" != util_props.rename_alpha_objs:
			if is_alp:
				if not obj.name[0].isalpha(): obj.name = "a" + obj.name
				obj.name = obj.name[0].lower() + obj.name[1:]
			else:
				obj.name = obj.name[0].upper() + obj.name[1:]
	
	bpy.context.scene.gta_tools.msg +=  "Finished : Successfully.\n"

def InitToolProps( str_props = [] ):
	print( "\n-----\n" + script_info + "\n-----" )
	print( "Initialize GTA Tools" )
	gta_tools = bpy.context.scene.gta_tools
	
	results = gta_tools.reset_tool_props( str_props )
	for res in results: gta_tools.set_msg( " Initialized : %s" %( res, ) )
	
	print( "-----" )
	gta_tools.set_msg( "Finished : Successfully." )


## Test Codes
def test01():
	#bpy.context.scene.gta_tools.reset_all_props()
	bpy.context.scene.gta_tools.reset_tool_props( "ifp_props" )

def test02():
	bpy.context.scene.gta_tools.msg  = ( "--- Test 02 ---" )
	bpy.context.scene.gta_tools.show_msg()

def test03():
	bpy.context.scene.gta_tools.msg  = ( "--- Test 03 ---" )
	bpy.context.scene.gta_tools.show_msg()


