# weight_tools.py @space_view3d_gta_tools
# 2011 ponz
script_info = "GTA Weight Tools ( built:2011.7.10 )"

# ***** BEGIN GPL LICENSE BLOCK *****
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or ( at your option ) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation, 
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ***** END GPL LICENCE BLOCK *****

#####
# ToDo.
#  - Graphical UI for Weight Gradient --- must study BGL API !!
#####


import bpy
import os
import time
import struct
from bpy.props import *
from mathutils import *
from math import *
#from rna_prop_ui import rna_idprop_ui_prop_get
import bgl
import blf


## Constants
# Grad Status Flags
GRAD_DRAW      = 0x01
GRAD_SET_1ST   = 0x02
GRAD_SET_2ND   = 0x04


## Classes


## Variables
callbacks = {}


## Functions
def vw_set_1st():
	weight_props = bpy.context.scene.gta_tools.weight_props
	weight_props.cur_loc_1st = bpy.context.scene.cursor_location
	if not ( weight_props.grad_flags & GRAD_DRAW ):
		weight_props.grad_flags |= GRAD_DRAW | GRAD_SET_1ST
		set_callbacks()
	bpy.context.area.tag_redraw()
	
	print( "-----\nGradient 1st point" )
	print( "%.3f, %.3f, %.3f" %( weight_props.cur_loc_1st[0:3] ) )
	
def vw_set_2nd():
	weight_props = bpy.context.scene.gta_tools.weight_props
	str_msg = ""
	
	mobj = bpy.context.active_object
	m = mobj.data
	
	weight_props.cur_loc_2nd = bpy.context.scene.cursor_location
	#pview_mat = bpy.context.region_data.perspective_matrix
	view_mat = bpy.context.region_data.view_matrix.to_3x3()
	
	view_loc_1st = Vector( weight_props.cur_loc_1st ) * view_mat
	view_loc_2nd = Vector( weight_props.cur_loc_2nd ) * view_mat
	view_loc_1st.z = 0
	view_loc_2nd.z = 0
	vec_grad = view_loc_2nd - view_loc_1st
	vec_grad /= ( vec_grad.length**2 )
	
	print( "-----\nGradient 2nd point" )
	print( "%.3f, %.3f, %.3f" %( weight_props.cur_loc_2nd[0:3] ) )
	weight_props.grad_flags |= GRAD_SET_2ND
	bpy.context.area.tag_redraw()
	

def vw_grad_clear():
	weight_props = bpy.context.scene.gta_tools.weight_props
	callbacks_clear()
	weight_props.grad_flags = 0


def vw_grad():
	weight_props = bpy.context.scene.gta_tools.weight_props
	str_msg = ""
	
	mobj = bpy.context.active_object
	m = mobj.data
	
	str_msg += ( "----- Gradient Weights ---\n" )
	
	view_mat = bpy.context.region_data.view_matrix.to_3x3()
	view_loc_1st = Vector( weight_props.cur_loc_1st ) * view_mat
	view_loc_2nd = Vector( weight_props.cur_loc_2nd ) * view_mat
	view_loc_1st.z = 0
	view_loc_2nd.z = 0
	vec_grad = view_loc_2nd - view_loc_1st
	vec_grad /= ( vec_grad.length**2 )
	
	active_vg = mobj.vertex_groups.active
	w_start = weight_props.grad_range[0]
	w_end   = weight_props.grad_range[1]
	contour = weight_props.grad_contour
	
	bpy.ops.object.mode_set( mode = 'OBJECT' )
	sel_verts = [ v for v in m.vertices if v.select ]
	
	weights = []
	for v in sel_verts:
		view_loc = v.co * view_mat
		view_loc.z = 0
		vec = view_loc - view_loc_1st
		
		if 'LINE' == contour:
			weight = vec.dot( vec_grad )
		
		elif 'CIRCLE' == contour:
			weight = vec.length * vec_grad.length
		
		weight = max( weight, 0.0 )
		weight = min( weight, 1.0 )
		weight = (1 - weight ) * w_start + weight * w_end
		
		if 0.001 > weight:
			active_vg.remove( [v.index] )
		else:
			active_vg.add( [v.index], weight, 'REPLACE' )
	
	bpy.ops.object.mode_set( mode = 'EDIT' )
	vw_grad_clear()
	if weight_props.grad_and_norm: vw_normalize()
	
	str_msg += ( "  Re-Assign Weights : %d verts\n" %( len( sel_verts ) ) )
	bpy.context.scene.gta_tools.msg = str_msg


def vw_clear():
	weight_props = bpy.context.scene.gta_tools.weight_props
	
	mobj = bpy.context.active_object
	m = mobj.data
	
	bpy.ops.object.mode_set( mode = 'OBJECT' )
	sel_verts = [ v for v in m.vertices if v.select ]
	
	print( "-----\nClear Weights" )
	for v in sel_verts:
		for vvg in v.groups:
			vg = mobj.vertex_groups[vvg.group]
			vg.remove( [v.index] )
	print( "  Clear Weights : " + str( len( sel_verts ) ) + " verts" )
	
	bpy.ops.object.mode_set( mode = 'EDIT' )
	
	print( "-----\nDone" )

def vw_clear_zero():
	weight_props = bpy.context.scene.gta_tools.weight_props
	
	mobj = bpy.context.active_object
	m = mobj.data
	
	bpy.ops.object.mode_set( mode = 'OBJECT' )
	sel_verts = [ v for v in m.vertices if v.select ]
	num_zero = 0
	
	print( "-----\nClear Zero Weights" )
	for v in sel_verts:
		for vvg in v.groups:
			vg = mobj.vertex_groups[vvg.group]
			if 0.001 > vvg.weight:
				vg.remove( [v.index] )
				num_zero += 1
	
	print( "  Clear Zero : " + str( num_zero ) + " verts" )
	
	bpy.ops.object.mode_set( mode = 'EDIT' )
	
	print( "-----\nDone" )

def vw_normalize():
	weight_props = bpy.context.scene.gta_tools.weight_props
	
	mobj = bpy.context.active_object
	m = mobj.data
	
	bpy.ops.object.mode_set( mode = 'OBJECT' )
	sel_verts = [ v for v in m.vertices if v.select ]
	
	print( "-----\nNormalize" )
	for v in sel_verts:
		vw_total = 0
		for vvg in v.groups:
			vg = mobj.vertex_groups[vvg.group]
			if 0.001 <= vvg.weight:
				vw_total += vvg.weight
			else:
				vg.remove( [v.index] )
		
		for vvg in v.groups:
			vvg.weight /= vw_total
	
	print( "  Normalized : " + str( len( sel_verts ) ) + " verts" )
	
	bpy.ops.object.mode_set( mode = 'EDIT' )
	
	print( "-----\nDone" )

def vw_select():
	weight_props = bpy.context.scene.gta_tools.weight_props
	
	mobj = bpy.context.active_object
	m = mobj.data
	
	bpy.ops.object.mode_set( mode = 'OBJECT' )
	sel_verts = [ v for v in m.vertices if v.select ]
	if 0 == len( sel_verts ): sel_verts = m.vertices
	
	print( "-----\nSelect" )
	for v in sel_verts:
		v.select = 3 < len( v.groups )
	
	sel_verts = [ v for v in m.vertices if v.select ]
	print( "  Selected :  " + str( len( sel_verts ) ) + " verts" )
	
	bpy.ops.object.mode_set( mode = 'EDIT' )
	
	print( "-----\nDone" )


def get_vert_group():
	# Active vtx group
	mobj = bpy.context.active_object
	aobj = mobj.modifiers[0].object
	active_vg   = mobj.vertex_groups.active
	active_bone_pos = ( aobj.matrix_world * aobj.data.bones.get(active_vg.name).matrix_local ).to_translation()
	
	return active_vg.name, active_bone_pos
	
def get_grad_color( var, a ):  # var: a scaler value range[0:1]
	return var,0,0, a

def get_heat4f( h, a ):  # h: a scaler value range[0:1]
	hi = floor( h*6 )
	f = h*6 - hi
	q = 1-f
	t = f
	
	if   0 == hi : r, g, b = 1  , f  , 0
	elif 1 == hi : r, g, b = 1-f, 1  , 0
	elif 2 == hi : r, g, b = 0  , 1  , f
	elif 3 == hi : r, g, b = 0  , 1-f, 1
	elif 4 == hi : r, g, b = f  , 0  , 1
	else         : r, g, b = 1  , 0  , 1-f
	
	return r, g, b, a


## Callbacks for BGL Drawing

def set_callbacks():
	global callbacks
	context = bpy.context
	region = context.region
	region_id = hash(region)
	cb_data = callbacks.get(region_id)
	if cb_data is None:
		hcb_px   = region.callback_add(draw_callback_px  , (context,), 'POST_PIXEL')
		hcb_view = region.callback_add(draw_callback_view, (context,), 'POST_VIEW')
		callbacks[region_id] = region, hcb_px, hcb_view
		print( "-----\nEnabled Draw CallBack" )
	else:
		region.callback_remove(cb_data[1])
		region.callback_remove(cb_data[2])
		del callbacks[region_id]
		print( "-----\nDisabled Draw CallBack" )
	
	context.area.tag_redraw()
	return {'FINISHED'}

def callbacks_clear():
	global callbacks
	for region, hcb_px, hcb_view in callbacks.values():
		region.callback_remove(hcb_px)
		region.callback_remove(hcb_view)
	callbacks.clear()
	print( "-----\nDisabled Draw CallBack" )
	bpy.context.area.tag_redraw()


def draw_callback_px(context):
	weight_props = context.scene.gta_tools.weight_props
	font_id = 0  # XXX, need to find out how best to get this.
	region = context.region
	region3d = context.space_data.region_3d
	region_mid_width  = region.width  / 2.0
	region_mid_height = region.height / 2.0
	perspective_matrix = region3d.perspective_matrix.copy()
	
	def draw_text(text, vec):
		vec_4d = vec.to_4d()
		vec_4d *= perspective_matrix
		if vec_4d.w > 0.0:
			x = region_mid_width  + region_mid_width * (vec_4d.x / vec_4d.w)
			y = region_mid_height + region_mid_height * (vec_4d.y / vec_4d.w)
			blf.position(font_id, x + 3.0, y - 4.0, 0.0)
			blf.draw(font_id, text)
	
	if not ( weight_props.grad_flags & GRAD_DRAW ): return
	
	# draw Caption
	blf.position(font_id, 22, region.height - 30, 0 )
	blf.draw(font_id, "Weight Gradient")
	
	# draw Active VG name
	active_vg = get_vert_group()
	draw_text( active_vg[0], active_vg[1] )
	
	if weight_props.grad_flags & GRAD_SET_1ST :
		# 1st Point
		loc_1st = Vector( weight_props.cur_loc_1st )
		draw_text( "1st", loc_1st )
		
		if weight_props.grad_flags & GRAD_SET_2ND:
			# 2nd Point
			loc_2nd = Vector( weight_props.cur_loc_2nd )
			draw_text( "2nd", loc_2nd )


def draw_callback_view(context):
	weight_props = context.scene.gta_tools.weight_props
	if not ( weight_props.grad_flags & GRAD_DRAW ): return
	
	bgl.glEnable(bgl.GL_BLEND)
	bgl.glShadeModel(bgl.GL_SMOOTH)
	
	# Active VG Position
	active_vg = get_vert_group()
	bgl.glPointSize(10.0)
	bgl.glBegin(bgl.GL_POINTS)
	bgl.glColor4f(1.0, 1.0, 1.0, 0.5)
	bgl.glVertex3f( *active_vg[1] )
	bgl.glEnd()
	
	if weight_props.grad_flags & GRAD_SET_1ST :
		# 1st Point
		col_1st = get_grad_color( weight_props.grad_range[0], 0.5 )
		col_2nd = get_grad_color( weight_props.grad_range[1], 0.5 )
		loc_1st = weight_props.cur_loc_1st
		bgl.glPointSize(10.0)
		bgl.glBegin(bgl.GL_POINTS)
		bgl.glColor4f( *col_1st )
		bgl.glVertex3f( *loc_1st )
		bgl.glEnd()
		
		if weight_props.grad_flags & GRAD_SET_2ND:
			# 2nd Point
			loc_2nd = weight_props.cur_loc_2nd
			bgl.glPointSize(10.0)
			bgl.glBegin(bgl.GL_POINTS)
			bgl.glColor4f( *col_2nd )
			bgl.glVertex3f( *loc_2nd )
			bgl.glEnd()
			
		else:
			loc_2nd = bpy.context.scene.cursor_location
		
		bgl.glLineWidth(2)
		bgl.glBegin(bgl.GL_LINE_STRIP)
		bgl.glColor4f( *col_1st )
		bgl.glVertex3f( *loc_1st )
		bgl.glColor4f( *col_2nd )
		bgl.glVertex3f( *loc_2nd )
		bgl.glEnd()


	# restore opengl defaults
	bgl.glLineWidth(1)
	bgl.glDisable(bgl.GL_BLEND)
	bgl.glColor4f(0.0, 0.0, 0.0, 1.0)


